@@ -1,5 +1,5 @@
 /*
-   $Id: dbdimp.h,v 0.6 1996/06/07 03:01:38 mhm Rel $
+   $Id: dbdimp.h,v 0.7 1996/10/07 18:00:05 mhm Rel $
 
    Copyright (c) 1995,1996 International Business Machines Corp.
 */
@@ -28,15 +28,15 @@ struct imp_sth_st {
     dbih_stc_t com;				/* MUST be first element in structure	*/
 	SQLHSTMT	phstmt;
     /* Input Details	*/
-    char	*statement;  		/* sql (see sth_scan)		*/
+    SQLCHAR	*statement;  		/* sql (see sth_scan)		*/
     HV		*bind_names;
 
     /* Output Details		*/
-    int        done_desc;  		/* have we described this sth yet ?	*/
+    SQLINTEGER        done_desc;  		/* have we described this sth yet ?	*/
     imp_fbh_t *fbh;	    		/* array of imp_fbh_t structs	*/
-    char      *fbh_cbuf;    	/* memory for all field names       */
-    int		long_buflen;      	/* length for long/longraw (if >0)	*/
-    char 	long_trunc_ok;    	/* is truncating a long an error	*/
+    SQLCHAR      *fbh_cbuf;    	/* memory for all field names       */
+    SQLINTEGER		long_buflen;      	/* length for long/longraw (if >0)	*/
+    SQLCHAR 	long_trunc_ok;    	/* is truncating a long an error	*/
 };
 #define IMP_STH_EXECUTING	0x0001
 
@@ -45,11 +45,11 @@ struct imp_fbh_st { 	/* field buffer EXPERIMENTAL */
     imp_sth_t *imp_sth;	/* 'parent' statement */
 
     /* description of the field	*/
-    int  dbsize;
+    SQLINTEGER  dbsize;
     short  dbtype;
-    char	*cbuf;		/* ptr to name of select-list item */
-    int  cbufl;			/* length of select-list item name */
-    int  dsize;			/* max display size if field is a char */
+    SQLCHAR	*cbuf;		/* ptr to name of select-list item */
+    SQLINTEGER  cbufl;			/* length of select-list item name */
+    SQLINTEGER  dsize;			/* max display size if field is a SQLCHAR */
     unsigned long prec;
     short  scale;
     short  nullok;
@@ -57,9 +57,9 @@ struct imp_fbh_st { 	/* field buffer EXPERIMENTAL */
     /* Our storage space for the field data as it's fetched	*/
     short ftype;		/* external datatype we wish to get	*/
     short  indp;		/* null/trunc indicator variable	*/
-    char	*buf;		/* data buffer (points to sv data)	*/
+    SQLCHAR	*buf;		/* data buffer (poSQLINTEGERs to sv data)	*/
     short  bufl;		/* length of data buffer		*/
-    int rlen;		/* length of returned data		*/
+    SQLINTEGER rlen;		/* length of returned data		*/
     short  rcode;		/* field level error status		*/
 
     SV	*sv;
@@ -71,17 +71,17 @@ typedef struct phs_st phs_t;    /* scalar placeholder   */
 struct phs_st {	/* scalar placeholder EXPERIMENTAL	*/
     SV	*sv;		/* the scalar holding the value		*/
     short ftype;	/* external OCI field type		*/
-    int indp;		/* null indicator or length indicator */
+    SQLINTEGER indp;		/* null indicator or length indicator */
 };
 
-char sql_state[6];
+SQLCHAR sql_state[6];
 
-void	do_error _((SV *h,int rc, SQLHENV henv, SQLHDBC hconn, 
-					SQLHSTMT hstmt, char *what));
-void	fbh_dump _((imp_fbh_t *fbh, int i));
+void	do_error _((SV *h,SQLINTEGER rc, SQLHENV henv, SQLHDBC hconn, 
+					SQLHSTMT hstmt, SQLCHAR *what));
+void	fbh_dump _((imp_fbh_t *fbh, SQLINTEGER i));
 
 void	dbd_init _((dbistate_t *dbistate));
-void	dbd_preparse _((imp_sth_t *imp_sth, char *statement));
-int	dbd_describe _((SV *h, imp_sth_t *imp_sth));
+void	dbd_preparse _((imp_sth_t *imp_sth, SQLCHAR *statement));
+SQLINTEGER	dbd_describe _((SV *h, imp_sth_t *imp_sth));
 
 /* end */
